@@ -55,6 +55,7 @@ from mycli.packages.filepaths import dir_path_exists, guess_socket_location
 from mycli.packages.hybrid_redirection import get_redirect_components, is_redirect_command
 from mycli.packages.parseutils import is_destructive, is_dropping_database
 from mycli.packages.prompt_utils import confirm, confirm_destructive_query
+from mycli.packages.special.favoritequeries import FavoriteQueries
 from mycli.packages.special.main import ArgType
 from mycli.packages.tabular_output import sql_format
 from mycli.packages.toolkit.history import FileHistoryWithTimestamp
@@ -132,6 +133,8 @@ class MyCli:
         special.set_timing_enabled(c["main"].as_bool("timing"))
         self.beep_after_seconds = float(c["main"]["beep_after_seconds"] or 0)
 
+        FavoriteQueries.instance = FavoriteQueries.from_config(self.config)
+
         self.dsn_alias: str | None = None
         self.main_formatter = TabularOutputFormatter(format_name=c["main"]["table_format"])
         self.redirect_formatter = TabularOutputFormatter(format_name=c["main"].get("redirect_format", "csv"))
@@ -805,7 +808,9 @@ class MyCli:
                         return
                     except special.FinishIteration as e:
                         if e.results:
-                            output_res(e.results, start)
+                            return output_res(e.results, start)
+                        else:
+                            return None
                     except RuntimeError as e:
                         logger.error("sql: %r, error: %r", text, e)
                         logger.error("traceback: %r", traceback.format_exc())
