@@ -1,22 +1,32 @@
+from __future__ import annotations
+
 import logging
 import os
 import platform
+
+from pymysql import ProgrammingError
+from pymysql.cursors import Cursor
+
 from mycli import __version__
 from mycli.packages.special import iocommands
+from mycli.packages.special.main import ArgType, special_command
 from mycli.packages.special.utils import format_uptime
-from .main import special_command, RAW_QUERY, PARSED_QUERY
-from pymysql import ProgrammingError
 
-log = logging.getLogger(__name__)
+logger = logging.getLogger(__name__)
 
 
-@special_command("\\dt", "\\dt[+] [table]", "List or describe tables.", arg_type=PARSED_QUERY, case_sensitive=True)
-def list_tables(cur, arg=None, arg_type=PARSED_QUERY, verbose=False):
+@special_command("\\dt", "\\dt[+] [table]", "List or describe tables.", arg_type=ArgType.PARSED_QUERY, case_sensitive=True)
+def list_tables(
+    cur: Cursor,
+    arg: str | None = None,
+    _arg_type: ArgType = ArgType.PARSED_QUERY,
+    verbose: bool = False,
+) -> list[tuple]:
     if arg:
-        query = "SHOW FIELDS FROM {0}".format(arg)
+        query = f'SHOW FIELDS FROM {arg}'
     else:
         query = "SHOW TABLES"
-    log.debug(query)
+    logger.debug(query)
     cur.execute(query)
     tables = cur.fetchall()
     status = ""
@@ -26,18 +36,19 @@ def list_tables(cur, arg=None, arg_type=PARSED_QUERY, verbose=False):
         return [(None, None, None, "")]
 
     if verbose and arg:
-        query = "SHOW CREATE TABLE {0}".format(arg)
-        log.debug(query)
+        query = f'SHOW CREATE TABLE {arg}'
+        logger.debug(query)
         cur.execute(query)
-        status = cur.fetchone()[1]
+        if one := cur.fetchone():
+            status = one[1]
 
     return [(None, tables, headers, status)]
 
 
-@special_command("\\l", "\\l", "List databases.", arg_type=RAW_QUERY, case_sensitive=True)
-def list_databases(cur, **_):
+@special_command("\\l", "\\l", "List databases.", arg_type=ArgType.RAW_QUERY, case_sensitive=True)
+def list_databases(cur: Cursor, **_) -> list[tuple]:
     query = "SHOW DATABASES"
-    log.debug(query)
+    logger.debug(query)
     cur.execute(query)
     if cur.description:
         headers = [x[0] for x in cur.description]
@@ -46,21 +57,23 @@ def list_databases(cur, **_):
         return [(None, None, None, "")]
 
 
-@special_command("status", "\\s", "Get status information from the server.", arg_type=RAW_QUERY, aliases=("\\s",), case_sensitive=True)
-def status(cur, **_):
+@special_command(
+    "status", "\\s", "Get status information from the server.", arg_type=ArgType.RAW_QUERY, aliases=["\\s"], case_sensitive=True
+)
+def status(cur: Cursor, **_) -> list[tuple]:
     query = "SHOW GLOBAL STATUS;"
-    log.debug(query)
+    logger.debug(query)
     try:
         cur.execute(query)
     except ProgrammingError:
         # Fallback in case query fail, as it does with Mysql 4
         query = "SHOW STATUS;"
-        log.debug(query)
+        logger.debug(query)
         cur.execute(query)
     status = dict(cur.fetchall())
 
     query = "SHOW GLOBAL VARIABLES;"
-    log.debug(query)
+    logger.debug(query)
     cur.execute(query)
     variables = dict(cur.fetchall())
 
@@ -80,19 +93,21 @@ def status(cur, **_):
     implementation = platform.python_implementation()
     version = platform.python_version()
     client_info = []
-    client_info.append("mycli {0},".format(__version__))
-    client_info.append("running on {0} {1}".format(implementation, version))
+    client_info.append(f'mycli {__version__}')
+    client_info.append(f'running on {implementation} {version}')
     title.append(" ".join(client_info) + "\n")
 
     # Build the output that will be displayed as a table.
     output.append(("Connection id:", cur.connection.thread_id()))
 
     query = "SELECT DATABASE(), USER();"
-    log.debug(query)
+    logger.debug(query)
     cur.execute(query)
-    db, user = cur.fetchone()
-    if db is None:
+    if one := cur.fetchone():
+        db, user = one
+    else:
         db = ""
+        user = ""
 
     output.append(("Current database:", db))
     output.append(("Current user:", user))
@@ -106,20 +121,23 @@ def status(cur, **_):
         pager = "stdout"
     output.append(("Current pager:", pager))
 
-    output.append(("Server version:", "{0} {1}".format(variables["version"], variables["version_comment"])))
+    output.append(("Server version:", f'{variables["version"]} {variables["version_comment"]}'))
     output.append(("Protocol version:", variables["protocol_version"]))
 
     if "unix" in cur.connection.host_info.lower():
         host_info = cur.connection.host_info
     else:
-        host_info = "{0} via TCP/IP".format(cur.connection.host)
+        host_info = f'{cur.connection.host} via TCP/IP'
 
     output.append(("Connection:", host_info))
 
     query = "SELECT @@character_set_server, @@character_set_database, @@character_set_client, @@character_set_connection LIMIT 1;"
-    log.debug(query)
+    logger.debug(query)
     cur.execute(query)
-    charset = cur.fetchone()
+    if one := cur.fetchone():
+        charset = one
+    else:
+        charset = ("", "", "", "")
     output.append(("Server characterset:", charset[0]))
     output.append(("Db characterset:", charset[1]))
     output.append(("Client characterset:", charset[2]))
@@ -136,19 +154,19 @@ def status(cur, **_):
     if "Threads_connected" in status:
         # Print the current server statistics.
         stats = []
-        stats.append("Connections: {0}".format(status["Threads_connected"]))
+        stats.append(f'Connections: {status["Threads_connected"]}')
         if "Queries" in status:
-            stats.append("Queries: {0}".format(status["Queries"]))
-        stats.append("Slow queries: {0}".format(status["Slow_queries"]))
-        stats.append("Opens: {0}".format(status["Opened_tables"]))
+            stats.append(f'Queries: {status["Queries"]}')
+        stats.append(f'Slow queries: {status["Slow_queries"]}')
+        stats.append(f'Opens: {status["Opened_tables"]}')
         if "Flush_commands" in status:
-            stats.append("Flush tables: {0}".format(status["Flush_commands"]))
-        stats.append("Open tables: {0}".format(status["Open_tables"]))
+            stats.append(f'Flush tables: {status["Flush_commands"]}')
+        stats.append(f'Open tables: {status["Open_tables"]}')
         if "Queries" in status:
             queries_per_second = int(status["Queries"]) / int(status["Uptime"])
-            stats.append("Queries per second avg: {:.3f}".format(queries_per_second))
-        stats = "  ".join(stats)
-        footer.append("\n" + stats)
+            stats.append(f'Queries per second avg: {queries_per_second:.3f}')
+        stats_str = "  ".join(stats)
+        footer.append("\n" + stats_str)
 
     footer.append("--------------")
     return [("\n".join(title), output, "", "\n".join(footer))]
