@@ -1,3 +1,5 @@
+# type: ignore
+
 """Steps for behavioral style tests are defined in this module.
 
 Each step is defined by the string decorating it. This string is used
@@ -5,16 +7,15 @@ to call the step in "*.feature" file.
 
 """
 
+from behave import then, when
 import pexpect
-
 import wrappers
-from behave import when, then
 
 
 @when("we create database")
 def step_db_create(context):
     """Send create database."""
-    context.cli.sendline("create database {0};".format(context.conf["dbname_tmp"]))
+    context.cli.sendline(f"create database {context.conf['dbname_tmp']};")
 
     context.response = {"database_name": context.conf["dbname_tmp"]}
 
@@ -22,7 +23,7 @@ def step_db_create(context):
 @when("we drop database")
 def step_db_drop(context):
     """Send drop database."""
-    context.cli.sendline("drop database {0};".format(context.conf["dbname_tmp"]))
+    context.cli.sendline(f"drop database {context.conf['dbname_tmp']};")
 
 
 @when("we connect to test database")
@@ -30,7 +31,7 @@ def step_db_connect_test(context):
     """Send connect to database."""
     db_name = context.conf["dbname"]
     context.currentdb = db_name
-    context.cli.sendline("use {0};".format(db_name))
+    context.cli.sendline(f"use {db_name};")
 
 
 @when("we connect to quoted test database")
@@ -38,7 +39,7 @@ def step_db_connect_quoted_tmp(context):
     """Send connect to database."""
     db_name = context.conf["dbname"]
     context.currentdb = db_name
-    context.cli.sendline("use `{0}`;".format(db_name))
+    context.cli.sendline(f"use `{db_name}`;")
 
 
 @when("we connect to tmp database")
@@ -46,7 +47,7 @@ def step_db_connect_tmp(context):
     """Send connect to database."""
     db_name = context.conf["dbname_tmp"]
     context.currentdb = db_name
-    context.cli.sendline("use {0}".format(db_name))
+    context.cli.sendline(f"use {db_name}")
 
 
 @when("we connect to dbserver")
@@ -68,7 +69,7 @@ def step_see_prompt(context):
     user = context.conf["user"]
     host = context.conf["host"]
     dbname = context.currentdb
-    wrappers.wait_prompt(context, "{0}@{1}:{2}> ".format(user, host, dbname))
+    wrappers.wait_prompt(context, f"{user}@{host}:{dbname}> ")
 
 
 @then("we see help output")
@@ -98,7 +99,7 @@ def step_see_db_dropped_no_default(context):
     context.currentdb = None
 
     wrappers.expect_exact(context, "Query OK, 0 rows affected", timeout=2)
-    wrappers.wait_prompt(context, "{0}@{1}:{2}>".format(user, host, database))
+    wrappers.wait_prompt(context, f"{user}@{host}:{database}>")
 
 
 @then("we see database connected")
@@ -106,4 +107,4 @@ def step_see_db_connected(context):
     """Wait to see drop database output."""
     wrappers.expect_exact(context, 'You are now connected to database "', timeout=2)
     wrappers.expect_exact(context, '"', timeout=2)
-    wrappers.expect_exact(context, ' as user "{0}"'.format(context.conf["user"]), timeout=2)
+    wrappers.expect_exact(context, f' as user "{context.conf["user"]}"', timeout=2)
